@@ -86,6 +86,13 @@ typedef enum{
 	SalProtoRtpSavp
 }SalMediaProto;
 
+typedef enum{
+	SalStreamSendRecv,
+	SalStreamSendOnly,
+	SalStreamRecvOnly,
+	SalStreamInactive
+}SalStreamDir;
+
 typedef struct SalEndpointCandidate{
 	char addr[64];
 	int port;
@@ -96,12 +103,14 @@ typedef struct SalEndpointCandidate{
 typedef struct SalStreamDescription{
 	SalMediaProto proto;
 	SalStreamType type;
+	char typeother[32];
 	char addr[64];
 	int port;
 	MSList *payloads; //<list of PayloadType
 	int bandwidth;
 	int ptime;
 	SalEndpointCandidate candidates[SAL_ENDPOINT_CANDIDATE_MAX];
+	SalStreamDir dir;
 } SalStreamDescription;
 
 #define SAL_MEDIA_DESCRIPTION_MAX_STREAMS 4
@@ -112,15 +121,20 @@ typedef struct SalMediaDescription{
 	char username[64];
 	int nstreams;
 	int bandwidth;
+	unsigned int session_ver;
+	unsigned int session_id;
 	SalStreamDescription streams[SAL_MEDIA_DESCRIPTION_MAX_STREAMS];
 } SalMediaDescription;
 
 SalMediaDescription *sal_media_description_new();
 void sal_media_description_ref(SalMediaDescription *md);
 void sal_media_description_unref(SalMediaDescription *md);
-bool_t sal_media_description_empty(SalMediaDescription *md);
-const SalStreamDescription *sal_media_description_find_stream(const SalMediaDescription *md,
+bool_t sal_media_description_empty(const SalMediaDescription *md);
+bool_t sal_media_description_equals(const SalMediaDescription *md1, const SalMediaDescription *md2);
+bool_t sal_media_description_has_dir(const SalMediaDescription *md, SalStreamDir dir);
+SalStreamDescription *sal_media_description_find_stream(SalMediaDescription *md,
     SalMediaProto proto, SalStreamType type);
+void sal_media_description_set_dir(SalMediaDescription *md, SalStreamDir stream_dir);
 
 /*this structure must be at the first byte of the SalOp structure defined by implementors*/
 typedef struct SalOpBase{
@@ -130,6 +144,7 @@ typedef struct SalOpBase{
 	char *from;
 	char *to;
 	char *origin;
+	char *remote_ua;
 	SalMediaDescription *local_media;
 	SalMediaDescription *remote_media;
 	void *user_pointer;
@@ -177,9 +192,10 @@ typedef void (*SalOnCallReceived)(SalOp *op);
 typedef void (*SalOnCallRinging)(SalOp *op);
 typedef void (*SalOnCallAccepted)(SalOp *op);
 typedef void (*SalOnCallAck)(SalOp *op);
-typedef void (*SalOnCallUpdated)(SalOp *op);
+typedef void (*SalOnCallUpdating)(SalOp *op);/*< Called when a reINVITE is received*/
 typedef void (*SalOnCallTerminated)(SalOp *op, const char *from);
-typedef void (*SalOnCallFailure)(SalOp *op, SalError error, SalReason reason, const char *details);
+typedef void (*SalOnCallFailure)(SalOp *op, SalError error, SalReason reason, const char *details, int code);
+typedef void (*SalOnCallReleased)(SalOp *salop);
 typedef void (*SalOnAuthRequested)(SalOp *op, const char *realm, const char *username);
 typedef void (*SalOnAuthSuccess)(SalOp *op, const char *realm, const char *username);
 typedef void (*SalOnRegisterSuccess)(SalOp *op, bool_t registered);
@@ -200,9 +216,10 @@ typedef struct SalCallbacks{
 	SalOnCallRinging call_ringing;
 	SalOnCallAccepted call_accepted;
 	SalOnCallAck call_ack;
-	SalOnCallUpdated call_updated;
+	SalOnCallUpdating call_updating;
 	SalOnCallTerminated call_terminated;
 	SalOnCallFailure call_failure;
+	SalOnCallReleased call_released;
 	SalOnAuthRequested auth_requested;
 	SalOnAuthSuccess auth_success;
 	SalOnRegisterSuccess register_success;
@@ -226,6 +243,10 @@ typedef struct SalAuthInfo{
 	char *realm;
 }SalAuthInfo;
 
+SalAuthInfo* sal_auth_info_new();
+SalAuthInfo* sal_auth_info_clone(const SalAuthInfo* auth_info);
+void sal_auth_info_delete(const SalAuthInfo* auth_info);
+
 void sal_set_callbacks(Sal *ctx, const SalCallbacks *cbs);
 int sal_listen_port(Sal *ctx, const char *addr, int port, SalTransport tr, int is_secure);
 int sal_unlisten_ports(Sal *ctx);
@@ -233,7 +254,14 @@ ortp_socket_t sal_get_socket(Sal *ctx);
 void sal_set_user_agent(Sal *ctx, const char *user_agent);
 /*keepalive period in ms*/
 void sal_set_keepalive_period(Sal *ctx,unsigned int value);
+/**
+ * returns keepalive period in ms
+ * 0 desactiaved
+ * */
+unsigned int sal_get_keepalive_period(Sal *ctx);
 void sal_use_session_timers(Sal *ctx, int expires);
+void sal_use_double_registrations(Sal *ctx, bool_t enabled);
+void sal_use_one_matching_codec_policy(Sal *ctx, bool_t one_matching_codec);
 int sal_iterate(Sal *sal);
 MSList * sal_get_pending_auths(Sal *sal);
 
@@ -248,6 +276,7 @@ void sal_op_set_from(SalOp *op, const char *from);
 void sal_op_set_to(SalOp *op, const char *to);
 void sal_op_release(SalOp *h);
 void sal_op_authenticate(SalOp *h, const SalAuthInfo *info);
+void sal_op_cancel_authentication(SalOp *h);
 void sal_op_set_user_pointer(SalOp *h, void *up);
 int sal_op_get_auth_requested(SalOp *h, const char **realm, const char **username);
 const char *sal_op_get_from(const SalOp *op);
@@ -257,20 +286,30 @@ const char *sal_op_get_route(const SalOp *op);
 const char *sal_op_get_proxy(const SalOp *op);
 /*for incoming requests, returns the origin of the packet as a sip uri*/
 const char *sal_op_get_network_origin(const SalOp *op);
+/*returns far-end "User-Agent" string */
+const char *sal_op_get_remote_ua(const SalOp *op);
 void *sal_op_get_user_pointer(const SalOp *op);
 
 /*Call API*/
 int sal_call_set_local_media_description(SalOp *h, SalMediaDescription *desc);
 int sal_call(SalOp *h, const char *from, const char *to);
-int sal_call_notify_ringing(SalOp *h);
+int sal_call_notify_ringing(SalOp *h, bool_t early_media);
+/*accept an incoming call or, during a call accept a reINVITE*/
 int sal_call_accept(SalOp*h);
 int sal_call_decline(SalOp *h, SalReason reason, const char *redirection /*optional*/);
+int sal_call_update(SalOp *h, const char *subject);
 SalMediaDescription * sal_call_get_final_media_description(SalOp *h);
-int sal_refer(SalOp *h, const char *refer_to);
-int sal_refer_accept(SalOp *h);
+int sal_call_refer(SalOp *h, const char *refer_to);
+int sal_call_refer_with_replaces(SalOp *h, SalOp *other_call_h);
+int sal_call_accept_refer(SalOp *h);
+/*informs this call is consecutive to an incoming refer */
+int sal_call_set_referer(SalOp *h, SalOp *refered_call);
+/* returns the SalOp of a call that should be replaced by h, if any */
+SalOp *sal_call_get_replaces(SalOp *h);
 int sal_call_send_dtmf(SalOp *h, char dtmf);
 int sal_call_terminate(SalOp *h);
 bool_t sal_call_autoanswer_asked(SalOp *op);
+void sal_call_send_vfu_request(SalOp *h);
 
 /*Registration*/
 int sal_register(SalOp *op, const char *proxy, const char *from, int expires);
@@ -306,5 +345,4 @@ void __sal_op_init(SalOp *b, Sal *sal);
 void __sal_op_set_network_origin(SalOp *op, const char *origin /*a sip uri*/);
 void __sal_op_free(SalOp *b);
 
-
 #endif
