@@ -21,12 +21,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 
 #include "sal_eXosip2.h"
-
+#include "private.h"
 #include "offeranswer.h"
 
+static bool_t call_failure(Sal *sal, eXosip_event_t *ev);
+
 static void text_received(Sal *sal, eXosip_event_t *ev);
 
-static void _osip_list_set_empty(osip_list_t *l, void (*freefunc)(void*)){
+void _osip_list_set_empty(osip_list_t *l, void (*freefunc)(void*)){
 	void *data;
 	while((data=osip_list_get(l,0))!=NULL){
 		osip_list_remove(l,0);
@@ -102,7 +104,7 @@ static SalOp * sal_find_other(Sal *sal, osip_message_t *response){
 	return NULL;
 }
 
-static void sal_add_other(Sal *sal, SalOp *op, osip_message_t *request){
+void sal_add_other(Sal *sal, SalOp *op, osip_message_t *request){
 	osip_call_id_t *callid=osip_message_get_call_id(request);
 	if (callid==NULL) {
 		ms_error("There is no call id in the request !");
@@ -161,8 +163,11 @@ SalOp * sal_op_new(Sal *sal){
 	op->sdp_answer=NULL;
 	op->reinvite=FALSE;
 	op->call_id=NULL;
+	op->replaces=NULL;
+	op->referred_by=NULL;
 	op->masquerade_via=FALSE;
 	op->auto_answer_asked=FALSE;
+	op->auth_info=NULL;
 	return op;
 }
 
@@ -201,6 +206,15 @@ void sal_op_release(SalOp *op){
 		sal_remove_other(op->base.root,op);
 		osip_call_id_free(op->call_id);
 	}
+	if (op->replaces){
+		ms_free(op->replaces);
+	}
+	if (op->referred_by){
+		ms_free(op->referred_by);
+	}
+	if (op->auth_info) {
+		sal_auth_info_delete(op->auth_info);
+	}
 	__sal_op_free(op);
 }
 
@@ -252,6 +266,7 @@ Sal * sal_init(){
 	eXosip_init();
 	sal=ms_new0(Sal,1);
 	sal->keepalive_period=30;
+	sal->double_reg=TRUE;
 	return sal;
 }
 
@@ -284,8 +299,10 @@ void sal_set_callbacks(Sal *ctx, const SalCallbacks *cbs){
 		ctx->callbacks.call_failure=(SalOnCallFailure)unimplemented_stub;
 	if (ctx->callbacks.call_terminated==NULL) 
 		ctx->callbacks.call_terminated=(SalOnCallTerminated)unimplemented_stub;
-	if (ctx->callbacks.call_updated==NULL) 
-		ctx->callbacks.call_updated=(SalOnCallUpdated)unimplemented_stub;
+	if (ctx->callbacks.call_released==NULL)
+		ctx->callbacks.call_released=(SalOnCallReleased)unimplemented_stub;
+	if (ctx->callbacks.call_updating==NULL) 
+		ctx->callbacks.call_updating=(SalOnCallUpdating)unimplemented_stub;
 	if (ctx->callbacks.auth_requested==NULL) 
 		ctx->callbacks.auth_requested=(SalOnAuthRequested)unimplemented_stub;
 	if (ctx->callbacks.auth_success==NULL) 
@@ -322,13 +339,17 @@ int sal_listen_port(Sal *ctx, const char *addr, int port, SalTransport tr, int i
 	int err;
 	bool_t ipv6;
 	int proto=IPPROTO_UDP;
+	int keepalive = ctx->keepalive_period;
 	
 	switch (tr) {
 	case SalTransportDatagram:
 		proto=IPPROTO_UDP;
+		eXosip_set_option (EXOSIP_OPT_UDP_KEEP_ALIVE, &keepalive);	
 		break;
 	case SalTransportStream:
 		proto= IPPROTO_TCP;
+			keepalive=-1;	
+		eXosip_set_option (EXOSIP_OPT_UDP_KEEP_ALIVE,&keepalive);	
 		break;
 	default:
 		ms_warning("unexpected proto, using datagram");
@@ -349,7 +370,7 @@ int sal_listen_port(Sal *ctx, const char *addr, int port, SalTransport tr, int i
 #ifdef HAVE_EXOSIP_GET_SOCKET
 	ms_message("Exosip has socket number %i",eXosip_get_socket(proto));
 #endif
-	eXosip_set_option (EXOSIP_OPT_UDP_KEEP_ALIVE, &ctx->keepalive_period);
+	
 	ctx->running=TRUE;
 	return err;
 }
@@ -371,10 +392,18 @@ void sal_use_session_timers(Sal *ctx, int expires){
 	ctx->session_expires=expires;
 }
 
+void sal_use_one_matching_codec_policy(Sal *ctx, bool_t one_matching_codec){
+	ctx->one_matching_codec=one_matching_codec;
+}
+
 MSList *sal_get_pending_auths(Sal *sal){
 	return ms_list_copy(sal->pending_auths);
 }
 
+void sal_use_double_registrations(Sal *ctx, bool_t enabled){
+	ctx->double_reg=enabled;
+}
+
 static int extract_received_rport(osip_message_t *msg, const char **received, int *rportval){
 	osip_via_t *via=NULL;
 	osip_generic_param_t *param=NULL;
@@ -441,8 +470,10 @@ static void sdp_process(SalOp *h){
 		offer_answer_initiate_outgoing(h->base.local_media,h->base.remote_media,h->result);
 	}else{
 		int i;
-		offer_answer_initiate_incoming(h->base.local_media,h->base.remote_media,h->result);
+		offer_answer_initiate_incoming(h->base.local_media,h->base.remote_media,h->result,h->base.root->one_matching_codec);
 		h->sdp_answer=media_description_to_sdp(h->result);
+		/*once we have generated the SDP answer, we modify the result description for processing by the upper layer.
+		 It should contains media parameters constraint from the remote offer, not our response*/
 		strcpy(h->result->addr,h->base.remote_media->addr);
 		h->result->bandwidth=h->base.remote_media->bandwidth;
 		for(i=0;i<h->result->nstreams;++i){
@@ -490,6 +521,12 @@ int sal_call(SalOp *h, const char *from, const char *to){
 		h->sdp_offering=TRUE;
 		set_sdp_from_desc(invite,h->base.local_media);
 	}else h->sdp_offering=FALSE;
+	if (h->replaces){
+		osip_message_set_header(invite,"Replaces",h->replaces);
+		if (h->referred_by)
+			osip_message_set_header(invite,"Referred-By",h->referred_by);
+	}
+	
 	eXosip_lock();
 	err=eXosip_call_send_initial_invite(invite);
 	eXosip_unlock();
@@ -503,10 +540,31 @@ int sal_call(SalOp *h, const char *from, const char *to){
 	return 0;
 }
 
-int sal_call_notify_ringing(SalOp *h){
-	eXosip_lock();
-	eXosip_call_send_answer(h->tid,180,NULL);
-	eXosip_unlock();
+int sal_call_notify_ringing(SalOp *h, bool_t early_media){
+	osip_message_t *msg;
+	int err;
+	
+	/*if early media send also 180 and 183 */
+	if (early_media && h->sdp_answer){
+		msg=NULL;
+		eXosip_lock();
+		err=eXosip_call_build_answer(h->tid,180,&msg);
+		if (msg){
+			set_sdp(msg,h->sdp_answer);
+			eXosip_call_send_answer(h->tid,180,msg);
+		}
+		msg=NULL;
+		err=eXosip_call_build_answer(h->tid,183,&msg);
+		if (msg){
+			set_sdp(msg,h->sdp_answer);
+			eXosip_call_send_answer(h->tid,183,msg);
+		}
+		eXosip_unlock();
+	}else{
+		eXosip_lock();
+		eXosip_call_send_answer(h->tid,180,NULL);
+		eXosip_unlock();
+	}
 	return 0;
 }
 
@@ -585,11 +643,20 @@ SalMediaDescription * sal_call_get_final_media_description(SalOp *h){
 	return h->result;
 }
 
+int sal_call_set_referer(SalOp *h, SalOp *refered_call){
+	if (refered_call->replaces)
+		h->replaces=ms_strdup(refered_call->replaces);
+	if (refered_call->referred_by)
+		h->referred_by=ms_strdup(refered_call->referred_by);
+	return 0;
+}
+
 int sal_ping(SalOp *op, const char *from, const char *to){
 	osip_message_t *options=NULL;
 	
 	sal_op_set_from(op,from);
 	sal_op_set_to(op,to);
+	/*bug here: eXosip2 does not honor the route argument*/
 	eXosip_options_build_request (&options, sal_op_get_to(op),
 			sal_op_get_from(op),sal_op_get_route(op));
 	if (options){
@@ -603,7 +670,7 @@ int sal_ping(SalOp *op, const char *from, const char *to){
 	return -1;
 }
 
-int sal_refer_accept(SalOp *op){
+int sal_call_accept_refer(SalOp *op){
 	osip_message_t *msg=NULL;
 	int err=0;
 	eXosip_lock();
@@ -623,7 +690,7 @@ int sal_refer_accept(SalOp *op){
 	return err;
 }
 
-int sal_refer(SalOp *h, const char *refer_to){
+int sal_call_refer(SalOp *h, const char *refer_to){
 	osip_message_t *msg=NULL;
 	int err=0;
 	eXosip_lock();
@@ -634,6 +701,38 @@ int sal_refer(SalOp *h, const char *refer_to){
 	return err;
 }
 
+int sal_call_refer_with_replaces(SalOp *h, SalOp *other_call_h){
+	osip_message_t *msg=NULL;
+	char referto[256]={0};
+	int err=0;
+	eXosip_lock();
+	if (eXosip_call_get_referto(other_call_h->did,referto,sizeof(referto)-1)!=0){
+		ms_error("eXosip_call_get_referto() failed for did=%i",other_call_h->did);
+		eXosip_unlock();
+		return -1;
+	}
+	eXosip_call_build_refer(h->did,referto, &msg);
+	osip_message_set_header(msg,"Referred-By",h->base.from);
+	if (msg) err=eXosip_call_send_request(h->did, msg);
+	else err=-1;
+	eXosip_unlock();
+	return err;
+}
+
+SalOp *sal_call_get_replaces(SalOp *h){
+	if (h->replaces!=NULL){
+		int cid;
+		eXosip_lock();
+		cid=eXosip_call_find_by_replaces(h->replaces);
+		eXosip_unlock();
+		if (cid>0){
+			SalOp *ret=sal_find_call(h->base.root,cid);
+			return ret;
+		}
+	}
+	return NULL;
+}
+
 int sal_call_send_dtmf(SalOp *h, char dtmf){
 	osip_message_t *msg=NULL;
 	char dtmf_body[128];
@@ -653,34 +752,59 @@ int sal_call_send_dtmf(SalOp *h, char dtmf){
 	return 0;
 }
 
+static void push_auth_to_exosip(const SalAuthInfo *info){
+	const char *userid;
+	if (info->userid==NULL || info->userid[0]=='\0') userid=info->username;
+	else userid=info->userid;
+	ms_message("Authentication info for username [%s], id[%s], realm [%s] added to eXosip", info->username,userid, info->realm);
+	eXosip_add_authentication_info (info->username,userid,
+                                  info->password, NULL,info->realm);
+}
+/*
+ * Just for symmetry ;-)
+ */
+static void pop_auth_from_exosip() {
+	eXosip_clear_authentication_info();
+}
+
 int sal_call_terminate(SalOp *h){
+	int err;
+	if (h->auth_info) push_auth_to_exosip(h->auth_info);
 	eXosip_lock();
-	eXosip_call_terminate(h->cid,h->did);
+	err=eXosip_call_terminate(h->cid,h->did);
 	eXosip_unlock();
-	sal_remove_call(h->base.root,h);
-	h->cid=-1;
+	pop_auth_from_exosip();
+	if (err!=0){
+		ms_warning("Exosip could not terminate the call: cid=%i did=%i", h->cid,h->did);
+	}
 	return 0;
 }
 
 void sal_op_authenticate(SalOp *h, const SalAuthInfo *info){
 	if (h->pending_auth){
-		const char *userid;
-		if (info->userid==NULL || info->userid[0]=='\0') userid=info->username;
-		else userid=info->userid;
-		ms_message("Authentication info for %s %s added to eXosip", info->username,info->realm);
-		eXosip_add_authentication_info (info->username,userid,
-                                      info->password, NULL,info->realm);
+		push_auth_to_exosip(info);
 		eXosip_lock();
 		eXosip_default_action(h->pending_auth);
 		eXosip_unlock();
 		ms_message("eXosip_default_action() done");
-		eXosip_clear_authentication_info();
+		pop_auth_from_exosip();
 		eXosip_event_free(h->pending_auth);
 		sal_remove_pending_auth(sal_op_get_sal(h),h);
 		h->pending_auth=NULL;
+		if (h->auth_info) sal_auth_info_delete(h->auth_info); /*if already exist*/
+		h->auth_info=sal_auth_info_clone(info); /*store auth info for subsequent request*/
 	}
 }
+void sal_op_cancel_authentication(SalOp *h) {
+	if (h->rid >0) {
+		sal_op_get_sal(h)->callbacks.register_failure(h,SalErrorFailure, SalReasonForbidden,_("Authentication failure"));
+	} else if (h->cid >0) {
+		sal_op_get_sal(h)->callbacks.call_failure(h,SalErrorFailure, SalReasonForbidden,_("Authentication failure"),0);
+	} else {
+		ms_warning("Auth failure not handled");
+	}
 
+}
 static void set_network_origin(SalOp *op, osip_message_t *req){
 	const char *received=NULL;
 	int rport=5060;
@@ -697,6 +821,31 @@ static void set_network_origin(SalOp *op, osip_message_t *req){
 	__sal_op_set_network_origin(op,origin);
 }
 
+static void set_remote_ua(SalOp* op, osip_message_t *req){
+	if (op->base.remote_ua==NULL){
+		osip_header_t *h=NULL;
+		osip_message_get_user_agent(req,0,&h);
+		if (h){
+			op->base.remote_ua=ms_strdup(h->hvalue);
+		}
+	}
+}
+
+static void set_replaces(SalOp *op, osip_message_t *req){
+	osip_header_t *h=NULL;
+
+	if (op->replaces){
+		ms_free(op->replaces);
+		op->replaces=NULL;
+	}
+	osip_message_header_get_byname(req,"replaces",0,&h);
+	if (h){
+		if (h->hvalue && h->hvalue[0]!='\0'){
+			op->replaces=ms_strdup(h->hvalue);
+		}
+	}
+}
+
 static SalOp *find_op(Sal *sal, eXosip_event_t *ev){
 	if (ev->cid>0){
 		return sal_find_call(sal,ev->cid);
@@ -704,6 +853,12 @@ static SalOp *find_op(Sal *sal, eXosip_event_t *ev){
 	if (ev->rid>0){
 		return sal_find_register(sal,ev->rid);
 	}
+	if (ev->sid>0){
+		return sal_find_out_subscribe(sal,ev->sid);
+	}
+	if (ev->nid>0){
+		return sal_find_in_subscribe(sal,ev->nid);
+	}
 	if (ev->response) return sal_find_other(sal,ev->response);
 	return NULL;
 }
@@ -716,6 +871,8 @@ static void inc_new_call(Sal *sal, eXosip_event_t *ev){
 	sdp_message_t *sdp=eXosip_get_sdp_info(ev->request);
 
 	set_network_origin(op,ev->request);
+	set_remote_ua(op,ev->request);
+	set_replaces(op,ev->request);
 	
 	if (sdp){
 		op->sdp_offering=FALSE;
@@ -769,35 +926,27 @@ static void handle_reinvite(Sal *sal,  eXosip_event_t *ev){
 		sal_media_description_unref(op->base.remote_media);
 		op->base.remote_media=NULL;
 	}
-	eXosip_lock();
-	eXosip_call_build_answer(ev->tid,200,&msg);
-	eXosip_unlock();
-	if (msg==NULL) return;
-	if (op->base.root->session_expires!=0){
-		if (op->supports_session_timers) osip_message_set_supported(msg, "timer");
-	}
-	if (op->base.contact){
-		_osip_list_set_empty(&msg->contacts,(void (*)(void*))osip_contact_free);
-		osip_message_set_contact(msg,op->base.contact);
+	if (op->result){
+		sal_media_description_unref(op->result);
+		op->result=NULL;
 	}
 	if (sdp){
 		op->sdp_offering=FALSE;
 		op->base.remote_media=sal_media_description_new();
 		sdp_to_media_description(sdp,op->base.remote_media);
 		sdp_message_free(sdp);
-		sdp_process(op);
-		if (op->sdp_answer!=NULL){
-			set_sdp(msg,op->sdp_answer);
-			sdp_message_free(op->sdp_answer);
-			op->sdp_answer=NULL;
-		}
+		sal->callbacks.call_updating(op);
 	}else {
 		op->sdp_offering=TRUE;
-		set_sdp_from_desc(msg,op->base.local_media);
+		eXosip_lock();
+		eXosip_call_build_answer(ev->tid,200,&msg);
+		if (msg!=NULL){
+			set_sdp_from_desc(msg,op->base.local_media);
+			eXosip_call_send_answer(ev->tid,200,msg);
+		}
+		eXosip_unlock();
 	}
-	eXosip_lock();
-	eXosip_call_send_answer(ev->tid,200,msg);
-	eXosip_unlock();
+	
 }
 
 static void handle_ack(Sal *sal,  eXosip_event_t *ev){
@@ -816,7 +965,7 @@ static void handle_ack(Sal *sal,  eXosip_event_t *ev){
 		sdp_message_free(sdp);
 	}
 	if (op->reinvite){
-		sal->callbacks.call_updated(op);
+		if (sdp) sal->callbacks.call_updating(op);
 		op->reinvite=FALSE;
 	}else{
 		sal->callbacks.call_ack(op);
@@ -840,6 +989,7 @@ static void update_contact_from_response(SalOp *op, osip_message_t *response){
 			tmp=sal_address_as_string(addr);
 			ms_message("Contact address updated to %s for this dialog",tmp);
 			sal_op_set_contact(op,tmp);
+			sal_address_destroy(addr);
 			ms_free(tmp);
 		}
 	}
@@ -855,7 +1005,8 @@ static int call_proceeding(Sal *sal, eXosip_event_t *ev){
 		eXosip_unlock();
 		return -1;
 	}
-	op->did=ev->did;
+	if (ev->did>0)
+		op->did=ev->did;
 	op->tid=ev->tid;
 	
 	/* update contact if received and rport are set by the server
@@ -868,7 +1019,8 @@ static void call_ringing(Sal *sal, eXosip_event_t *ev){
 	sdp_message_t *sdp;
 	SalOp *op=find_op(sal,ev);
 	if (call_proceeding(sal, ev)==-1) return;
-	
+
+	set_remote_ua(op,ev->response);
 	sdp=eXosip_get_sdp_info(ev->response);
 	if (sdp){
 		op->base.remote_media=sal_media_description_new();
@@ -891,7 +1043,8 @@ static void call_accepted(Sal *sal, eXosip_event_t *ev){
 	}
 
 	op->did=ev->did;
-	
+	set_remote_ua(op,ev->response);
+
 	sdp=eXosip_get_sdp_info(ev->response);
 	if (sdp){
 		op->base.remote_media=sal_media_description_new();
@@ -924,8 +1077,6 @@ static void call_terminated(Sal *sal, eXosip_event_t *ev){
 	if (ev->request){
 		osip_from_to_str(ev->request->from,&from);
 	}
-	sal_remove_call(sal,op);
-	op->cid=-1;
 	sal->callbacks.call_terminated(op,from!=NULL ? from : sal_op_get_from(op));
 	if (from) osip_free(from);
 }
@@ -936,9 +1087,11 @@ static void call_released(Sal *sal, eXosip_event_t *ev){
 		ms_warning("No op associated to this call_released()");
 		return;
 	}
-	op->cid=-1;
-	if (op->did==-1) 
-		sal->callbacks.call_failure(op,SalErrorNoResponse,SalReasonUnknown,NULL);
+	if (ev->response==NULL){
+		/* no response received so far */
+		call_failure(sal,ev);
+	}
+	sal->callbacks.call_released(op);
 }
 
 static int get_auth_data_from_response(osip_message_t *resp, const char **realm, const char **username){
@@ -1100,13 +1253,40 @@ static bool_t call_failure(Sal *sal, eXosip_event_t *ev){
 				sr=SalReasonUnknown;
 			}else error=SalErrorNoResponse;
 	}
-	sal->callbacks.call_failure(op,error,sr,reason);
+	sal->callbacks.call_failure(op,error,sr,reason,code);
 	if (computedReason != NULL){
 		ms_free(computedReason);
 	}
 	return TRUE;
 }
 
+/* Request remote side to send us VFU */
+void sal_call_send_vfu_request(SalOp *h){
+	osip_message_t *msg=NULL;
+	char info_body[] =
+			"<?xml version=\"1.0\" encoding=\"utf-8\" ?>"
+			 "<media_control>"
+			 "  <vc_primitive>"
+			 "    <to_encoder>"
+			 "      <picture_fast_update></picture_fast_update>"
+			 "    </to_encoder>"
+			 "  </vc_primitive>"
+			 "</media_control>";
+
+	char clen[10];
+
+	eXosip_lock();
+	eXosip_call_build_info(h->did,&msg);
+	if (msg){
+		osip_message_set_body(msg,info_body,strlen(info_body));
+		osip_message_set_content_type(msg,"application/media_control+xml");
+		snprintf(clen,sizeof(clen),"%lu",(unsigned long)strlen(info_body));
+		osip_message_set_content_length(msg,clen);
+		eXosip_call_send_request(h->did,msg);
+		ms_message("Sending VFU request !");
+	}
+	eXosip_unlock();
+}
 
 static void process_media_control_xml(Sal *sal, eXosip_event_t *ev){
 	SalOp *op=find_op(sal,ev);
@@ -1127,8 +1307,18 @@ static void process_media_control_xml(Sal *sal, eXosip_event_t *ev){
 			eXosip_call_build_answer(ev->tid,200,&ans);
 			if (ans)
 				eXosip_call_send_answer(ev->tid,200,ans);
+			return;
 		}
 	}
+	/*in all other cases we must say it is not implemented.*/
+	{
+		osip_message_t *ans=NULL;
+		eXosip_lock();
+		eXosip_call_build_answer(ev->tid,501,&ans);
+		if (ans)
+			eXosip_call_send_answer(ev->tid,501,ans);
+		eXosip_unlock();
+	}
 }
 
 static void process_dtmf_relay(Sal *sal, eXosip_event_t *ev){
@@ -1156,9 +1346,64 @@ static void process_dtmf_relay(Sal *sal, eXosip_event_t *ev){
 					sal->callbacks.dtmf_received(op, tmp[0]);
 			}
 		}
+		eXosip_lock();
 		eXosip_call_build_answer(ev->tid,200,&ans);
 		if (ans)
 			eXosip_call_send_answer(ev->tid,200,ans);
+		eXosip_unlock();
+	}
+}
+
+static void fill_options_answer(osip_message_t *options){
+	osip_message_set_allow(options,"INVITE, ACK, BYE, CANCEL, OPTIONS, MESSAGE, SUBSCRIBE, NOTIFY, INFO");
+	osip_message_set_accept(options,"application/sdp");
+}
+
+static void process_refer(Sal *sal, SalOp *op, eXosip_event_t *ev){
+	osip_header_t *h=NULL;
+	osip_message_t *ans=NULL;
+	ms_message("Receiving REFER request !");
+	osip_message_header_get_byname(ev->request,"Refer-To",0,&h);
+
+	if (h){
+		osip_from_t *from=NULL;
+		char *tmp;
+		osip_from_init(&from);
+	
+		if (osip_from_parse(from,h->hvalue)==0){
+			if (op ){
+				osip_uri_header_t *uh=NULL;
+				osip_header_t *referred_by=NULL;
+				osip_uri_header_get_byname(&from->url->url_headers,(char*)"Replaces",&uh);
+				if (uh!=NULL && uh->gvalue && uh->gvalue[0]!='\0'){
+					ms_message("Found replaces in Refer-To");
+					if (op->replaces){
+						ms_free(op->replaces);
+					}
+					op->replaces=ms_strdup(uh->gvalue);
+				}
+				osip_message_header_get_byname(ev->request,"Referred-By",0,&referred_by);
+				if (referred_by && referred_by->hvalue && referred_by->hvalue[0]!='\0'){
+					if (op->referred_by)
+						ms_free(op->referred_by);
+					op->referred_by=ms_strdup(referred_by->hvalue);
+				}
+			}
+			osip_uri_header_freelist(&from->url->url_headers);
+			osip_from_to_str(from,&tmp);
+			sal->callbacks.refer_received(sal,op,tmp);
+			osip_free(tmp);
+			osip_from_free(from);
+		}
+		eXosip_lock();
+		eXosip_call_build_answer(ev->tid,202,&ans);
+		if (ans)
+			eXosip_call_send_answer(ev->tid,202,ans);
+		eXosip_unlock();
+	}
+	else
+	{
+		ms_warning("cannot do anything with the refer without destination\n");
 	}
 }
 
@@ -1190,8 +1435,7 @@ static void call_message_new(Sal *sal, eXosip_event_t *ev){
 					eXosip_call_send_answer(ev->tid,200,ans);
 				eXosip_unlock();
 			}
-		}
-		if(MSG_IS_MESSAGE(ev->request)){
+		}else if(MSG_IS_MESSAGE(ev->request)){
 			/* SIP messages could be received into call */
 			text_received(sal, ev);
 			eXosip_lock();
@@ -1199,27 +1443,12 @@ static void call_message_new(Sal *sal, eXosip_event_t *ev){
 			if (ans)
 				eXosip_call_send_answer(ev->tid,200,ans);
 			eXosip_unlock();
-		}
-		if(MSG_IS_REFER(ev->request)){
-			osip_header_t *h=NULL;
+		}else if(MSG_IS_REFER(ev->request)){
 			SalOp *op=find_op(sal,ev);
 			
 			ms_message("Receiving REFER request !");
-			osip_message_header_get_byname(ev->request,"Refer-To",0,&h);
-			eXosip_lock();
-			eXosip_call_build_answer(ev->tid,202,&ans);
-			if (ans)
-				eXosip_call_send_answer(ev->tid,202,ans);
-			eXosip_unlock();
-			if (h){
-				sal->callbacks.refer_received(sal,op,h->hvalue);
-			}
-			else
-			{
-				ms_warning("cannot do anything with the refer without destination\n");
-			}
-		}
-		if(MSG_IS_NOTIFY(ev->request)){
+			process_refer(sal,op,ev);
+		}else if(MSG_IS_NOTIFY(ev->request)){
 			osip_header_t *h=NULL;
 			char *from=NULL;
 			SalOp *op=find_op(sal,ev);
@@ -1236,6 +1465,14 @@ static void call_message_new(Sal *sal, eXosip_event_t *ev){
 				eXosip_call_send_answer(ev->tid,200,ans);
 			eXosip_unlock();
 			osip_free(from);
+		}else if (MSG_IS_OPTIONS(ev->request)){
+			eXosip_lock();
+			eXosip_call_build_answer(ev->tid,200,&ans);
+			if (ans){
+				fill_options_answer(ans);
+				eXosip_call_send_answer(ev->tid,200,ans);
+			}
+			eXosip_unlock();
 		}
 	}else ms_warning("call_message_new: No request ?");
 }
@@ -1284,6 +1521,8 @@ static void text_received(Sal *sal, eXosip_event_t *ev){
 	osip_free(from);
 }
 
+
+
 static void other_request(Sal *sal, eXosip_event_t *ev){
 	ms_message("in other_request");
 	if (ev->request==NULL) return;
@@ -1293,8 +1532,7 @@ static void other_request(Sal *sal, eXosip_event_t *ev){
 	}else if (strcmp(ev->request->sip_method,"OPTIONS")==0){
 		osip_message_t *options=NULL;
 		eXosip_options_build_answer(ev->tid,200,&options);
-		osip_message_set_allow(options,"INVITE, ACK, BYE, CANCEL, OPTIONS, MESSAGE, SUBSCRIBE, NOTIFY, INFO");
-		osip_message_set_accept(options,"application/sdp");
+		fill_options_answer(options);
 		eXosip_options_send_answer(ev->tid,200,options);
 	}else if (strcmp(ev->request->sip_method,"WAKEUP")==0
 		&& comes_from_local_if(ev->request)) {
@@ -1304,12 +1542,7 @@ static void other_request(Sal *sal, eXosip_event_t *ev){
 	}else if (strncmp(ev->request->sip_method, "REFER", 5) == 0){
 		ms_message("Receiving REFER request !");
 		if (comes_from_local_if(ev->request)) {
-			osip_header_t *h=NULL;
-			osip_message_header_get_byname(ev->request,"Refer-To",0,&h);
-			eXosip_message_send_answer(ev->tid,200,NULL);
-			if (h){
-				sal->callbacks.refer_received(sal,NULL,h->hvalue);
-			}
+			process_refer(sal,NULL,ev);
 		}else ms_warning("Ignored REFER not coming from this local loopback interface.");
 	}else if (strncmp(ev->request->sip_method, "UPDATE", 6) == 0){
 		inc_update(sal,ev);
@@ -1345,6 +1578,9 @@ static bool_t register_again_with_updated_contact(SalOp *op, osip_message_t *ori
 	char *tmp;
 	char port[20];
 	SalAddress *addr;
+	Sal *sal=op->base.root;
+
+	if (sal->double_reg==FALSE) return FALSE;
 	
 	if (extract_received_rport(last_answer,&received,&rport)==-1) return FALSE;
 	osip_message_get_contact(orig_request,0,&ctt);
@@ -1405,7 +1641,9 @@ static void registration_success(Sal *sal, eXosip_event_t *ev){
 		if (!register_again_with_updated_contact(op,ev->request,ev->response)){
 			sal->callbacks.register_success(op,registered);
 		}
-	}else registered=FALSE;
+	}else {
+		sal->callbacks.register_success(op,FALSE);
+	}
 }
 
 static bool_t registration_failure(Sal *sal, eXosip_event_t *ev){
@@ -1422,7 +1660,7 @@ static bool_t registration_failure(Sal *sal, eXosip_event_t *ev){
 	if (ev->response){
 		status_code=osip_message_get_status_code(ev->response);
 		reason=osip_message_get_reason_phrase(ev->response);
-	}else return TRUE;
+	}
 	switch(status_code){
 		case 401:
 		case 407:
@@ -1436,7 +1674,7 @@ static bool_t registration_failure(Sal *sal, eXosip_event_t *ev){
 		default:
 			/* if contact is up to date, process the failure, otherwise resend a new register with
 				updated contact first, just in case the faillure is due to incorrect contact */
-			if (register_again_with_updated_contact(op,ev->request,ev->response))
+			if (ev->response && register_again_with_updated_contact(op,ev->request,ev->response))
 				return TRUE; /*we are retrying with an updated contact*/
 			if (status_code==403){
 				se=SalErrorFailure;
@@ -1519,7 +1757,7 @@ static bool_t process_event(Sal *sal, eXosip_event_t *ev){
 			call_message_new(sal,ev);
 			break;
 		case EXOSIP_CALL_MESSAGE_REQUESTFAILURE:
-			if (ev->did<0 && ev->response &&
+			if (ev->response &&
 				(ev->response->status_code==407 || ev->response->status_code==401)){
 				 return process_authentication(sal,ev);
 			}
@@ -1574,8 +1812,17 @@ static bool_t process_event(Sal *sal, eXosip_event_t *ev){
 			other_request_reply(sal,ev);
 			break;
 		case EXOSIP_MESSAGE_REQUESTFAILURE:
-			if (ev->response && (ev->response->status_code == 407 || ev->response->status_code == 401)){
-				return process_authentication(sal,ev);
+		case EXOSIP_NOTIFICATION_REQUESTFAILURE:
+			if (ev->response) {
+				switch (ev->response->status_code) {
+					case 407:
+					case 401:
+						return process_authentication(sal,ev);
+					case 412: {
+						eXosip_automatic_action ();
+						return 1;
+					}
+				}
 			}
 			other_request_reply(sal,ev);
 			break;
@@ -1764,10 +2011,15 @@ void sal_set_keepalive_period(Sal *ctx,unsigned int value) {
 	ctx->keepalive_period=value;
 	eXosip_set_option (EXOSIP_OPT_UDP_KEEP_ALIVE, &value);
 }
+unsigned int sal_get_keepalive_period(Sal *ctx) {
+	return ctx->keepalive_period;
+}
+
 const char * sal_address_get_port(const SalAddress *addr) {
 	const osip_from_t *u=(const osip_from_t*)addr;
 	return null_if_empty(u->url->port);
 }
+
 int sal_address_get_port_int(const SalAddress *uri) {
 	const char* port = sal_address_get_port(uri);
 	if (port != NULL) {
@@ -1777,3 +2029,29 @@ int sal_address_get_port_int(const SalAddress *uri) {
 	}
 }
 
+/* sends a reinvite. Local media description may have changed by application since call establishment*/
+int sal_call_update(SalOp *h, const char *subject){
+	int err=0;
+	osip_message_t *reinvite=NULL;
+
+	eXosip_lock();
+	if(eXosip_call_build_request(h->did,"INVITE",&reinvite) != 0 || reinvite==NULL){
+		eXosip_unlock();
+		return -1;
+	}
+	eXosip_unlock();
+	osip_message_set_subject(reinvite,subject);
+	osip_message_set_allow(reinvite, "INVITE, ACK, CANCEL, OPTIONS, BYE, REFER, NOTIFY, MESSAGE, SUBSCRIBE, INFO");
+	if (h->base.root->session_expires!=0){
+		osip_message_set_header(reinvite, "Session-expires", "200");
+		osip_message_set_supported(reinvite, "timer");
+	}
+	if (h->base.local_media){
+		h->sdp_offering=TRUE;
+		set_sdp_from_desc(reinvite,h->base.local_media);
+	}else h->sdp_offering=FALSE;
+	eXosip_lock();
+	err = eXosip_call_send_request(h->did, reinvite);
+	eXosip_unlock();
+	return err;
+}
