@@ -33,6 +33,12 @@ static MSSndCardManager * create_manager(){
 
 void ms_snd_card_manager_destroy(void){
 	if (scm!=NULL){
+		MSList *elem;
+		for(elem=scm->descs;elem!=NULL;elem=elem->next){
+			MSSndCardDesc *desc = (MSSndCardDesc*)elem->data;
+			if (desc->unload!=NULL)
+				desc->unload(scm);
+		}
 		ms_list_for_each(scm->cards,(void (*)(void*))ms_snd_card_destroy);
 		ms_list_free(scm->cards);
 		ms_list_free(scm->descs);
@@ -125,9 +131,13 @@ MSSndCard * ms_snd_card_dup(MSSndCard *card){
 }
 
 MSSndCard * ms_snd_card_new(MSSndCardDesc *desc){
+	return ms_snd_card_new_with_name(desc,NULL);
+}
+
+MSSndCard * ms_snd_card_new_with_name(MSSndCardDesc *desc,const char* name) {
 	MSSndCard *obj=(MSSndCard *)ms_new(MSSndCard,1);
 	obj->desc=desc;
-	obj->name=NULL;
+	obj->name=name?ms_strdup(name):NULL;
 	obj->data=NULL;
 	obj->id=NULL;
 	obj->capabilities=MS_SND_CARD_CAP_CAPTURE|MS_SND_CARD_CAP_PLAYBACK;
