@@ -34,6 +34,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <sys/types.h>
 #ifndef WIN32
 #include <unistd.h>
+#else
+#include <malloc.h>
 #endif
 #include <stdio.h>
 #include <stdlib.h>
@@ -42,8 +44,15 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 static int cond=1;
 
 static const char * capture_card=NULL;
+static const char * playback_card=NULL;
+static const char *infile=NULL,*outfile=NULL;
 static float ng_threshold=-1;
 static bool_t use_ng=FALSE;
+static bool_t two_windows=FALSE;
+
+/* starting values echo canceller */
+static int ec_len_ms=0, ec_delay_ms=250, ec_framesize;
+
 
 static void stop_handler(int signum)
 {
@@ -135,9 +144,13 @@ const char *usage="mediastream --local <port> --remote <ip:port> --payload <payl
 								"[ --agc (enable automatic gain control)]\n"
 								"[ --ng (enable noise gate)]\n"
 								"[ --ng-threshold <(float) [0-1]> (noise gate threshold)]\n"
-								"[ --capture-card <index>] \n";
-static void run_media_streams(int localport, const char *remote_ip, int remoteport, int payload, const char *fmtp, int jitter, bool_t ec, int bitrate, MSVideoSize vs, bool_t agc, bool_t eq);
+								"[ --capture-card <name>] \n"
+								"[ --playback-card <name>] \n"
+								"[ --infile	<input wav file>] specify a wav file to be used for input, instead of soundcard\n"
+								"[ --outfile <output wav file>] specify a wav file to write audio into, instead of soundcard\n";
 
+static void run_media_streams(int localport, const char *remote_ip, int remoteport, int payload, const char *fmtp,
+          int jitter, int bitrate, MSVideoSize vs, bool_t ec, bool_t agc, bool_t eq);
 
 int main(int argc, char * argv[])
 {
@@ -151,6 +164,7 @@ int main(int argc, char * argv[])
 	bool_t ec=FALSE;
 	bool_t agc=FALSE;
 	bool_t eq=FALSE;
+
 	/*create the rtp session */
 	ortp_init();
 	ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
@@ -158,7 +172,9 @@ int main(int argc, char * argv[])
 	rtp_profile_set_payload(&av_profile,110,&payload_type_speex_nb);
 	rtp_profile_set_payload(&av_profile,111,&payload_type_speex_wb);
 	rtp_profile_set_payload(&av_profile,112,&payload_type_ilbc);
+	rtp_profile_set_payload(&av_profile,113,&payload_type_amr);
 #ifdef VIDEO_ENABLED
+	rtp_profile_set_payload(&av_profile,26,&payload_type_jpeg);
 	rtp_profile_set_payload(&av_profile,98,&payload_type_h263_1998);
 	rtp_profile_set_payload(&av_profile,97,&payload_type_theora);
 	rtp_profile_set_payload(&av_profile,99,&payload_type_mp4v);
@@ -169,7 +185,7 @@ int main(int argc, char * argv[])
 	vs.width=MS_VIDEO_SIZE_CIF_W;
 	vs.height=MS_VIDEO_SIZE_CIF_H;
 	if (argc<4) {
-		printf(usage);
+		printf("%s",usage);
 		return -1;
 	}
 	for (i=1;i<argc;i++){
@@ -179,7 +195,7 @@ int main(int argc, char * argv[])
 		}else if (strcmp(argv[i],"--remote")==0){
 			i++;
 			if (!parse_addr(argv[i],ip,sizeof(ip),&remoteport)) {
-				printf(usage);
+				printf("%s",usage);
 				return -1;
 			}
 			printf("Remote addr: ip=%s port=%i\n",ip,remoteport);
@@ -204,6 +220,9 @@ int main(int argc, char * argv[])
 		}else if (strcmp(argv[i],"--capture-card")==0){
 			i++;
 			capture_card=argv[i];
+		}else if (strcmp(argv[i],"--playback-card")==0){
+			i++;
+			playback_card=argv[i];
 		}else if (strcmp(argv[i],"--ec")==0){
 			ec=TRUE;
 		}else if (strcmp(argv[i],"--agc")==0){
@@ -215,14 +234,23 @@ int main(int argc, char * argv[])
 		}else if (strcmp(argv[i],"--ng-threshold")==0){
 			i++;
 			ng_threshold=atof(argv[i]);
+		}else if (strcmp(argv[i],"--two-windows")==0){
+			two_windows=TRUE;
+		}else if (strcmp(argv[i],"--infile")==0){
+			i++;
+			infile=argv[i];
+		}else if (strcmp(argv[i],"--outfile")==0){
+			i++;
+			outfile=argv[i];
 		}
 	}
 
-	run_media_streams(localport,ip,remoteport,payload,fmtp,jitter,ec,bitrate,vs, agc,eq);
+	run_media_streams(localport,ip,remoteport,payload,fmtp,jitter,bitrate,vs,ec,agc,eq);
 	return 0;
 }
 
-void run_media_streams(int localport,  const char *remote_ip, int remoteport, int payload, const char *fmtp, int jitter, bool_t ec, int bitrate, MSVideoSize vs, bool_t agc, bool_t eq)
+static void run_media_streams(int localport, const char *remote_ip, int remoteport, int payload, const char *fmtp,
+          int jitter, int bitrate, MSVideoSize vs, bool_t ec, bool_t agc, bool_t eq)
 {
 	AudioStream *audio=NULL;
 #ifdef VIDEO_ENABLED
@@ -244,17 +272,20 @@ void run_media_streams(int localport,  const char *remote_ip, int remoteport, in
 	if (bitrate>0) pt->normal_bitrate=bitrate;
 
 	if (pt->type!=PAYLOAD_VIDEO){
-		printf("Starting audio stream.\n");
 		MSSndCardManager *manager=ms_snd_card_manager_get();
 		MSSndCard *capt= capture_card==NULL ? ms_snd_card_manager_get_default_capture_card(manager) :
 				ms_snd_card_manager_get_card(manager,capture_card);
+		MSSndCard *play= playback_card==NULL ? ms_snd_card_manager_get_default_playback_card(manager) :
+				ms_snd_card_manager_get_card(manager,playback_card);
 		audio=audio_stream_new(localport,ms_is_ipv6(remote_ip));
 		audio_stream_enable_automatic_gain_control(audio,agc);
 		audio_stream_enable_noise_gate(audio,use_ng);
-		audio_stream_start_now(audio,profile,remote_ip,remoteport,remoteport+1,payload,jitter,
-			ms_snd_card_manager_get_default_playback_card(manager),
-			capt,
-			 ec);
+		audio_stream_set_echo_canceller_params(audio,ec_len_ms,ec_delay_ms,ec_framesize);
+		printf("Starting audio stream.\n");
+	
+		audio_stream_start_full(audio,profile,remote_ip,remoteport,remoteport+1, payload, jitter,infile,outfile,
+		                        outfile==NULL ? play : NULL ,infile==NULL ? capt : NULL,infile!=NULL ? FALSE: ec);
+		
 		if (audio) {
 			if (use_ng && ng_threshold!=-1)
 				ms_filter_call_method(audio->volsend,MS_VOLUME_SET_NOISE_GATE_THRESHOLD,&ng_threshold);
@@ -269,6 +300,7 @@ void run_media_streams(int localport,  const char *remote_ip, int remoteport, in
 		printf("Starting video stream.\n");
 		video=video_stream_new(localport, ms_is_ipv6(remote_ip));
 		video_stream_set_sent_video_size(video,vs);
+		video_stream_use_preview_video_window(video,two_windows);
 		video_stream_start(video,profile,
 					remote_ip,
 					remoteport,remoteport+1,
@@ -280,7 +312,56 @@ void run_media_streams(int localport,  const char *remote_ip, int remoteport, in
 		printf("Error: video support not compiled.\n");
 #endif
 	}
-	if (!eq){
+  if (eq || ec){ /*read from stdin interactive commands */
+    char commands[128];
+    commands[127]='\0';
+    ms_sleep(1);  /* ensure following text be printed after ortp messages */
+    if (eq)
+      printf("\nPlease enter equalizer requests, such as 'eq active 1', 'eq active 0', 'eq 1200 0.1 200'\n");
+    if (ec)
+      printf("\nPlease enter echo canceller requests: ec reset; ec <delay ms> <tail_length ms'\n");
+    while(fgets(commands,sizeof(commands)-1,stdin)!=NULL){
+      int active,freq,freq_width;
+      int delay_ms, tail_ms;
+      float gain;
+      if (sscanf(commands,"eq active %i",&active)==1){
+        audio_stream_enable_equalizer(audio,active);
+        printf("OK\n");
+      }else if (sscanf(commands,"eq %i %f %i",&freq,&gain,&freq_width)==3){
+        audio_stream_equalizer_set_gain(audio,freq,gain,freq_width);
+        printf("OK\n");
+      }else if (sscanf(commands,"eq %i %f",&freq,&gain)==2){
+        audio_stream_equalizer_set_gain(audio,freq,gain,0);
+        printf("OK\n");
+      }else if (strstr(commands,"dump")){
+        int n=0,i;
+        float *t;
+        ms_filter_call_method(audio->equalizer,MS_EQUALIZER_GET_NUM_FREQUENCIES,&n);
+        t=(float*)alloca(sizeof(float)*n);
+        ms_filter_call_method(audio->equalizer,MS_EQUALIZER_DUMP_STATE,t);
+        for(i=0;i<n;++i){
+          if (fabs(t[i]-1)>0.01){
+            printf("%i:%f:0 ",(i*pt->clock_rate)/(2*n),t[i]);
+          }
+        }
+        printf("\nOK\n");
+      }else if (sscanf(commands,"ec reset %i",&active)==1){
+          //audio_stream_enable_equalizer(audio,active);
+          //printf("OK\n");
+      }else if (sscanf(commands,"ec active %i",&active)==1){
+          //audio_stream_enable_equalizer(audio,active);
+          //printf("OK\n");
+      }else if (sscanf(commands,"ec %i %i",&delay_ms,&tail_ms)==2){
+        audio_stream_set_echo_canceller_params(audio,tail_ms,delay_ms,128);
+        // revisit: workaround with old method call to force echo reset
+        delay_ms*=8;
+        ms_filter_call_method(audio->ec,MS_FILTER_SET_PLAYBACKDELAY,&delay_ms);
+        printf("OK\n");
+      }else if (strstr(commands,"quit")){
+        break;
+      }else printf("Cannot understand this.\n");
+    }
+	}else{  /* no interactive stuff - continuous debug output */
 		rtp_session_register_event_queue(session,q);
 		while(cond)
 		{
@@ -311,41 +392,9 @@ void run_media_streams(int localport,  const char *remote_ip, int remoteport, in
 				parse_events(q);
 			}
 		}
-	}else{/*read from stdin equalizer commands */
-		char commands[128];
-		commands[127]='\0';
-		printf("Please enter equalizer requests, such as 'eq active 1', 'eq active 0', 'eq 1200 0.1'\n");
-		while(fgets(commands,sizeof(commands)-1,stdin)!=NULL){
-			int active,freq,freq_width;
-			float gain;
-			if (sscanf(commands,"eq active %i",&active)==1){
-				audio_stream_enable_equalizer(audio,active);
-				printf("OK\n");
-			}else if (sscanf(commands,"eq %i:%f:%i",&freq,&gain,&freq_width)==3){
-				audio_stream_equalizer_set_gain(audio,freq,gain,freq_width);
-				printf("OK\n");
-			}else if (sscanf(commands,"eq %i:%f",&freq,&gain)==2){
-				audio_stream_equalizer_set_gain(audio,freq,gain,0);
-				printf("OK\n");
-			}else if (strstr(commands,"dump")){
-				int n=0,i;
-				float *t;
-				ms_filter_call_method(audio->equalizer,MS_EQUALIZER_GET_NUM_FREQUENCIES,&n);
-				t=(float*)alloca(sizeof(float)*n);
-				ms_filter_call_method(audio->equalizer,MS_EQUALIZER_DUMP_STATE,t);
-				for(i=0;i<n;++i){
-					if (fabs(t[i]-1)>0.01){
-						printf("%i:%f:0 ",(i*pt->clock_rate)/(2*n),t[i]);
 					}
-				}
-				printf("\nOK\n");
-			}else if (strstr(commands,"quit")){
-				break;
-			}else printf("Cannot understand this.\n");
-		}
-	}
 	
-	printf("stoping all...\n");
+	printf("stopping all...\n");
 	
 	if (audio) audio_stream_stop(audio);
 #ifdef VIDEO_ENABLED
